package GeneticCode::IUPAC;
BEGIN{ push(@INC, $ENV{LOCAL_PERL_MODULES}); }
use Carp;
use warnings;
use strict;
use GeneticCode::Tables;

my $code2bases = {
    'A' => ['A'], 
    'T' => ['T'], 
    'G' => ['G'], 
    'C' => ['C'], 
    'U' => ['T'], 
    'R' => ['A', 'G'], 
    'Y' => ['C', 'T'], 
    'M' => ['C', 'A'], 
    'K' => ['T', 'G'], 
    'W' => ['T', 'A'], 
    'S' => ['C', 'G'], 
    'B' => ['C', 'T', 'G'], 
    'D' => ['A', 'T', 'G'], 
    'H' => ['A', 'T', 'C'],
    'V' => ['A', 'C', 'G'], 
    'N' => ['A', 'T', 'G', 'C']
};

###################################################
sub isValidCodon {
    my $codon = shift;
    return 0 if length($codon) != 3;
    return 1 if $codon =~ /^[ATGCURYMKWSBDHVN]+$/;
    return 0;
}

###################################################
sub codon2aa {
    my $codon = shift;
    my $table = shift;
    my $codons = getAtgcCodons($codon);
    my $aa     = $table->{$codons->[0]};
    foreach my $_codon (@$codons){
	my $_aa = $table->{$_codon};
	return 'X' if $aa ne $_aa;
    }
    return $aa;
}

###################################################
sub getAtgcCodons {
    my $codon = shift;
    my $bases1 = $code2bases->{ substr($codon, 0, 1) };
    my $bases2 = $code2bases->{ substr($codon, 1, 1) };
    my $bases3 = $code2bases->{ substr($codon, 2, 1) };
    my $codons = [];
    foreach my $base1 (@$bases1){
	foreach my $base2 (@$bases2){
	    foreach my $base3 (@$bases3){
		my $_codon = $base1 . $base2 . $base3;
		push( @$codons, $_codon );
	    }
	}
    }
    return $codons;
}

1;
