package MetaGene::FileIO;
BEGIN{ push(@INC, $ENV{LOCAL_PERL_MODULES}); }
use Carp;
use warnings;
use strict;
use FASTA::Sequence;
use FASTA::Util;
use MetaGene::ORF;

###################################################
sub parse {
    my $filename = shift or croak "Input file name is not specified.";
    my $orfs = [];
    open IN, $filename or croak "Can't open $filename to read.";
    my $buffer = [];
    while(<IN>){
	chomp;
	if( !isEmptyLine($_) ){
	    push(@$buffer, $_);
	} else {
	    my $tmpORFs = getORFs( $buffer );
	    push(@$orfs, @$tmpORFs);
	    undef $buffer;
	}
    }
    close IN;
    return $orfs;
}

###################################################
sub isEmptyLine {
    my $line = shift;
    if( $line =~ /^\s*$/ ){
	return 1;
    } else {
	return 0;
    }
}

###################################################
sub getORFs {
    my $buffer = shift;
    my $title = getTitle($buffer);
    my $orfs = [];
    foreach my $line (@$buffer){
	next if( $line =~ /^#/ );
	my @cols = split("\t", $line);
	if( @cols == 6 ){
	    my $orf = MetaGene::ORF->new();
	    $orf->set( $title, \@cols );
	    push( @$orfs, $orf );
	}
    }
    return $orfs;
}

###################################################
sub getTitle {
    my $buffer = shift;
    my $titleLine = $buffer->[0];
    my $title = $1 if( $titleLine =~ /# (\S+)/ );
    return $title;
}


1;
