package MetaGene::Util;
BEGIN{ push(@INC, $ENV{LOCAL_PERL_MODULES}); }
use Carp;
use warnings;
use strict;
use FASTA::Sequence;
use MetaGene::ORF;
use GeneticCode::Util;

my $codeID = 11;

###################################################
sub translateORFs {
    my $orfs = shift or croak "ORFs are not specified.";
    my $seqs = shift or croak "Genome/Assembled sequences are not specified.";
    my $seqHash = FASTA::Util::indexing( $seqs );
    my $pepSeqs = [];
    foreach my $orf (@$orfs){
	my $pepSeq = translate( $orf, $seqHash );
	next if $pepSeq eq "";
	my $pepName = $orf->getORFName();
	my $faPepSeq = FASTA::Sequence->new();
	$faPepSeq->setTitle( $pepName );
	$faPepSeq->setSequence( $pepSeq );
	push( @$pepSeqs, $faPepSeq );
    }
    return $pepSeqs;
}

###################################################
sub translate {
    my $orf = shift;
    my $seqHash = shift;
    my $seqName = $orf->getSequenceName();
    my $wholeSeq = $seqHash->{$seqName};
    print $seqName . "\n" unless $seqHash->{$seqName};
    my $cds = $orf->getCDS( $wholeSeq );
    my $flgPartial5 = $orf->isPartial5();
    my $flgPartial3 = $orf->isPartial3();
    return "" unless $cds =~ /^[ATGC]+$/;
    my $pepSeq = GeneticCode::Util::translate( $cds, $codeID, $flgPartial5 );
    $pepSeq =~ s/\*$// unless( $flgPartial3 );
    if( $pepSeq =~ /\*/ ){
	die "Stop codon appears in CDS: "
	    . "\t" . $orf->getLine() . "\n"
	    . "\t" . $pepSeq . "\n";
    }
    return $pepSeq;
}


1;
