package BLAST::M0File;
BEGIN{ push(@INC, $ENV{LOCAL_PERL_MODULES}); }
use Carp;
use warnings;
use strict;
use BLAST::M0Hit;


###################################################
sub parse {
    my $file = shift;
    print "Parsing a BLAST result file in the M0 format...\n"
	. "\tfilename = " . $file . "\n";
    my $m0hits = [];
    open IN, $file or croak "Can't open $file to read.";
    my ($queryID, $subjectID);
    while(<IN>){
	chomp;
	$queryID   = $1 if /Query= (\S+)/;
	$subjectID = $1 if />(\S+)/;
	if( /Score =\s+([\d\.e\+]+) bits.+Expect =\s+([\d\.e\-]+)/ ){
	    my $hit = BLAST::M0Hit->new();
	    $hit->setQueryID( $queryID );
	    $hit->setSubjectID( $subjectID );
 	    my ($bitScore, $eValue) = ($1, $2);
 	    my $identityLine = <IN>; chomp $identityLine;
	    my $strandLine   = <IN>; chomp $strandLine;
	    my $alnString;
	    while(-1){
		my $blankLine   = <IN>; chomp $blankLine;
		$blankLine      = <IN>; chomp $blankLine;
		my $queryLine   = <IN>; chomp $queryLine;
		if( $queryLine =~ /Query= (\S+)/ ){ $queryID=$1; last; }
		if( $queryLine =~ />(\S+)/ ){ $subjectID=$1; last; }
		unless( $queryLine =~ /^Query:/ ){ last; }
		my $stateLine   = <IN>; chomp $stateLine;
		my $subjectLine = <IN>; chomp $subjectLine;
		$alnString     .= $queryLine . "\n"
		    . $stateLine   . "\n"
		    . $subjectLine . "\n" . "\n\n";
	    }
 	    $hit->setAlignmentScores( $eValue, $bitScore );
	    $hit->setAlignmentString( $alnString );
	    push( @$m0hits, $hit );
	}
    }
    close IN;
    print "\t" . scalar(@$m0hits) . " BLAST hits were loaded.\n"
	. "...done.\n\n";
    return $m0hits;
}


1;
